import { Database } from 'sqlite';
import { DatabaseInitializer } from '../initializeDatabase';
import GameState from '../../game/gameState';

/**
 * Service for interacting with the SQLite database for operations related to comments and user mentions.
 */
export class DatabaseService {
    /**
     * Retrieves the singleton instance of the database.
     * This static method ensures that a single database instance is used throughout the application,
     * following the singleton pattern for managing database connections.
     * 
     * @example
     * const db = await DatabaseService.getDatabase();
     * 
     * @returns {Promise<Database>} A promise that resolves to the initialized database instance.
     * @throws {Error} Will throw an error if the database cannot be initialized.
     */
    private static async getDatabase(): Promise<Database> {
        const databaseInitializer = DatabaseInitializer.getInstance();
        const db = await databaseInitializer.getDbInstance()
        if (!db) {
            throw new Error('Failed to initialize the database.');
        }
        return db
    }

    /**
     * Loads the active game state for a given author from the database.
     * If no active game state exists, it returns null.
     * 
     * @param {number} authorId - The unique identifier for the author whose game state is to be loaded.
     * @returns {Promise<GameState | null>} The loaded game state if it exists and is active, otherwise null.
     * @throws {Error} Throws an error if the database operation fails.
     * @example
     * const gameState = await DatabaseService.loadGameState(1);
     * if (gameState) {
     *     // Game state exists and is active
     * } else {
     *     // No active game state found
     * }
     */
    public static async loadGameState(authorId: number): Promise<GameState | null> {
        return this.withRetry(async () => {
            const db = await DatabaseService.getDatabase();
            const sql = `SELECT data FROM game_state WHERE authorId = ? AND active = TRUE`;
            const row = await db.get(sql, [authorId]);
            if (row) {
                return JSON.parse(row.data) as GameState;
            } else {
                return null;
            }
        });
    }

    /**
      * Saves the current game state to the database. If an active game state already exists for the author,
      * it updates that state. Otherwise, it creates a new active game state.
      * 
      * @param {number} authorId - The unique identifier for the author of the game state.
      * @param {GameState} gameState - The current state of the game to be saved.
      * @throws {Error} Throws an error if the database operation fails.
      * @example
      * await DatabaseService.saveGameState(1, currentGameState);
      */
    public static async saveGameState(authorId: number, gameState: GameState): Promise<void> {
        return this.withRetry(async () => {
            const db = await DatabaseService.getDatabase();
            const data = JSON.stringify(gameState);
            // Update the currently active game state
            const sql = `
            UPDATE game_state SET data = ?, modified_at = CURRENT_TIMESTAMP
            WHERE authorId = ? AND active = TRUE;
            `;
            const result = await db.run(sql, [data, authorId]);

            // Check if the update was successful (i.e., an active game state existed and was updated)
            if (result.changes === 0) {
                // No active game state was found to update, so insert a new active game state
                const insertSql = `
                INSERT INTO game_state (authorId, data, active) VALUES (?, ?, TRUE);
                `;
                await db.run(insertSql, [authorId, data]);
            }
        });
    }


    /**
     * Resets the game state for a given author by setting the active state to false.
     * This method should be called after a death event or when the player decides to start over.
     * 
     * @param {number} authorId - The unique identifier for the author of the game state to be reset.
     * @throws {Error} Throws an error if the database operation fails.
     * @example
     * await DatabaseService.resetGameState(1);
     */
    public static async resetGameState(authorId: number): Promise<void> {
        return this.withRetry(async () => {
            const db = await this.getDatabase();
            const sql = `UPDATE game_state SET active = FALSE WHERE authorId = ? AND active = TRUE`;
            await db.run(sql, [authorId]);
        });
    }

    /**
     * Attempts to execute a database operation with retries on failure.
     * This method is designed to handle SQLITE_BUSY errors by retrying the operation
     * with an exponential backoff strategy. This is particularly useful for handling
     * cases where the SQLite database is locked due to concurrent access attempts.
     *
     * @param operation A function that performs the database operation and returns a Promise.
     * @param maxRetries The maximum number of retries before giving up. Defaults to 5.
     * @param delay The initial delay (in milliseconds) before the first retry. This delay is doubled with each retry.
     * @returns A Promise that resolves with the result of the operation, or rejects if the operation fails after all retries.
     * @throws Will throw an error if the operation cannot be completed successfully within the allowed number of retries.
     */
    private static async withRetry<T>(operation: () => Promise<T>, maxRetries: number = 5, delay: number = 100): Promise<T> {
        for (let i = 0; i < maxRetries; i++) {
            try {
                return await operation();
            } catch (error) {
                console.log('Error', error)
                // Type check or assertion to handle the error as an object with a 'code' property
                const e = error as { code?: string };
                if (e.code === 'SQLITE_BUSY') {
                    console.log(`Database is busy, retrying... Attempt ${i + 1}`);
                    await new Promise(resolve => setTimeout(resolve, delay * Math.pow(2, i)));
                } else {
                    throw error; // Rethrow if it's not an SQLITE_BUSY error
                }
            }
        }
        throw new Error('Max retries reached for database operation');
    }
}
