import GameState, { DEATH_REASON_ENUM, PHASE_ENUM } from './gameState';
import { MessageService, MessageFileName } from '../utils/MessageService'
import { CommentPoster } from '../rdrama/services/CommentPoster';
import { RidersPhase } from '../phases/riders';
import { HuntPhase } from '../phases/hunt';
import { HandleSetup as SetupPhase } from '../phases/setup';
import { travelPhase as TravelPhase } from '../phases/travel';
import { ActionPhase } from '../phases/action';
import { EventPhase } from './gameEvents';
import { MountainPhase } from '../phases/mountain';

class GameFlow {

    static async executeCurrentPhase(gameState: GameState, userInput?: string) {
        GameFlow.cleanupGameState(gameState);

        switch (gameState.phase) {
            case PHASE_ENUM.SETUP:
                // Handle the setup phase, e.g., initial purchases
                await SetupPhase.handleSetupPhase(gameState, userInput);
                break;
            case PHASE_ENUM.START_TURN:
                await GameFlow.startTurn(gameState);
                break;
            case PHASE_ENUM.ACTION_CHOICE:
                await ActionPhase.actionChoice(gameState, userInput)
                break;
            case PHASE_ENUM.TRAVEL:
                // Handle travel logic
                await TravelPhase.handleTravel(gameState, userInput);
                break;
            case PHASE_ENUM.FORT:
                // Handle interactions at the fort
                await SetupPhase.handleSetupPhase(gameState, userInput);
                break;
            case PHASE_ENUM.HUNT:
                // Handle hunting logic
                await HuntPhase.handleHunt(gameState, userInput);
                break;
            case PHASE_ENUM.RIDERS_ATTACK:
                // Handle a riders attack phase
                await RidersPhase.handleRidersAttack(gameState, userInput);
                break;
            case PHASE_ENUM.EVENT:
                // Handle random events that can occur
                await EventPhase.handleRandomEvent(gameState);
                break;
            case PHASE_ENUM.MOUNTAIN:
                // Handle mountain traversal challenges
                await MountainPhase.handleMountain(gameState);
                break;
            case PHASE_ENUM.DEATH:
                // Check if conditions leading to the player's death have been met

                break;
            default:
                console.error("Unknown game phase:", gameState.phase);
                break;
        }
    }



    static cleanupGameState(gameState: GameState): void {
        // Ensure all expenditures are non-negative
        gameState.amountSpentOnFood = Math.max(0, gameState.amountSpentOnFood);
        gameState.amountSpentOnAmmunition = Math.max(0, gameState.amountSpentOnAmmunition);
        gameState.amountSpentOnClothing = Math.max(0, gameState.amountSpentOnClothing);
        gameState.amountSpentOnMiscellaneousSupplies = Math.max(0, gameState.amountSpentOnMiscellaneousSupplies);

        // Round expenditures and cash to integers
        gameState.amountSpentOnFood = Math.floor(gameState.amountSpentOnFood);
        gameState.amountSpentOnAmmunition = Math.floor(gameState.amountSpentOnAmmunition);
        gameState.amountSpentOnClothing = Math.floor(gameState.amountSpentOnClothing);
        gameState.amountSpentOnMiscellaneousSupplies = Math.floor(gameState.amountSpentOnMiscellaneousSupplies);
        gameState.totalMileageWholeTrip = Math.floor(gameState.totalMileageWholeTrip);
        //Round Cash to two digits
        gameState.cashLeftAfterInitialPurchases = Math.floor(gameState.cashLeftAfterInitialPurchases * 100) / 100;
    }

    static async startTurn(gameState: GameState): Promise<void> {
        // Update total mileage up through the previous turn
        gameState.totalMileageUpThroughPreviousTurn = gameState.totalMileageWholeTrip;

        // Toggle fort option flag
        gameState.fortOptionFlag = !gameState.fortOptionFlag;

        gameState.phase = PHASE_ENUM.ACTION_CHOICE;
        await gameState.save()

        GameFlow.executeCurrentPhase(gameState)
    }

    private async handleEncounter(): Promise<void> {
        // Logic for handling encounters
    }

    private async handleRandomEvent(): Promise<void> {
        // Logic for determining and handling a random event
    }

    private async handleMountain(): Promise<void> {
        // Logic for traversing the mountains, may affect gameState
    }


    static handleDoctor(gameState: GameState): string {
        let responseMessage: string = ""
        // Handle illness and injury

        if (gameState.illnessFlag && gameState.injuryFlag) {
            responseMessage += `You were sick and and injured `
        } else if (gameState.illnessFlag) {
            responseMessage += `You were sick and `
        } else {
            responseMessage += `You were injured and `
        }
        gameState.cashLeftAfterInitialPurchases -= 20;
        if (gameState.cashLeftAfterInitialPurchases < 0) {
            if (gameState.illnessFlag) gameState.death = DEATH_REASON_ENUM.ILLNESS
            if (gameState.injuryFlag) gameState.death = DEATH_REASON_ENUM.INJURY
            gameState.phase = PHASE_ENUM.DEATH
            return "";
        } else {

            responseMessage += `The Doctors bill was $20\n\n`
        }
        gameState.injuryFlag = false;
        gameState.illnessFlag = false;

        return responseMessage;

    }



}

export default GameFlow;

