import { DatabaseService } from "../db/services/Database";
import { Comment } from "../rdrama/models/Comment";

/**
 * Represents the state of a game session for an Oregon Trail-style game.
 * This class stores all relevant game state information and provides methods
 * to load from and save to a database.
 */
class GameState {
    comment: Comment;
    authorId: number;
    authorName: string;
    amountSpentOnAnimals: number = 0;
    amountSpentOnAmmunition: number = 0;
    shootResponseTime: number = 0;
    clockTimeAtStartOfBang: number = 0;
    amountSpentOnClothing: number = 0;
    insufficientClothingFlag: boolean = false;
    yesNoResponseToQuestions: string = '';
    eventCounter: number = 0;
    turnNumberForSettingDate: number = 0;
    currentDate: string = 'MONDAY MARCH 29 1847';
    shootingExpertiseLevelChoice: number = 0;
    eatingChoice: number = 0;
    amountSpentOnFood: number = 0;
    clearSouthPassFlag: boolean = false;
    clearBlueMountainsFlag: boolean = false;
    fractionOfTwoWeeksTraveledOnFinalTurn: number = 0;
    injuryFlag: boolean = false;
    blizzardFlag: boolean = false;
    totalMileageWholeTrip: number = 0;
    amountSpentOnMiscellaneousSupplies: number = 0;
    totalMileageUpThroughPreviousTurn: number = 0;
    clearSouthPassMileageFlag: boolean = false;
    amountSpentOnItemsAtFort: number = 0;
    randomEventNumber: number = 0;
    illnessFlag: boolean = false;
    hostileRiders: boolean = false;
    shootingWordSelector: string = '';
    shootingWordVariations: string[] = [];
    cashLeftAfterInitialPurchases: number = 800;
    tacticsChoiceWhenAttacked: number = 0;
    actionChoiceForEachTurn: number = 0;
    fortOptionFlag: boolean = false;
    phase: PHASE_ENUM = PHASE_ENUM.SETUP;
    subPhase: number = 0;
    death: DEATH_REASON_ENUM | undefined;
    deathReason: string | undefined;
    lastPrompt: string | undefined;

    private constructor(comment: Comment, state?: Partial<GameState>) {
        Object.assign(this, state); // Initialize with loaded state or undefined
        this.comment = comment
        this.authorId = comment.author_id;
        this.authorName = comment.author_name;
    }

    /**
    * Saves the current game state to the database.
    */
    public async save() {
        console.log(`Saving game state to database... Phase: ${this.phase} SubPhase: ${this.subPhase} Cash: ${this.cashLeftAfterInitialPurchases}`);
        await DatabaseService.saveGameState(this.authorId, this);
    }

    /**
     * Loads an existing game state from the database or creates a new one if it doesn't exist.
     * @param {Comment} comment - The invoking comment of the author/player.
     * @returns {Promise<GameState>} - The loaded or newly created game state.
     */
    public static async load(comment: Comment): Promise<GameState> {
        const loadedState = await DatabaseService.loadGameState(comment.author_id);
        if (loadedState) {
            return new GameState(comment, loadedState);
        } else {
            // Create a new GameState with default values
            const newState = new GameState(comment);
            await newState.save(); // Optionally save the new state to the database
            return newState;
        }
    }

    /**
     * Resets the current game state by saving the current state and then resetting the game state in the database.
     */
    public async reset(): Promise<GameState> {
        await DatabaseService.saveGameState(this.authorId, this);
        await DatabaseService.resetGameState(this.authorId)
        return GameState.load(this.comment)
    }

}

export default GameState

// Enumeration for different phases of the game.
export enum PHASE_ENUM {
    SETUP = 'SETUP',
    START_TURN = 'START_TURN',
    ACTION_CHOICE = 'ACTION_CHOICE',
    TRAVEL = 'TRAVEL',
    FORT = 'FORT',
    HUNT = 'HUNT',
    RIDERS_ATTACK = 'RIDERS_ATTACK',
    EVENT = 'EVENT',
    MOUNTAIN = 'MOUNTAIN',
    DEATH = 'DEATH',
}

export enum DEATH_REASON_ENUM {
    ILLNESS = 'Illness',
    INJURY = 'Injury',
    FREEZING = 'Freezing',
    STARVATION = 'Starvation',
    DROWNING = 'Drowning',
    WILDLIFE_ATTACK = 'Wildlife Attack',
    SNAKE_BITE = 'Snake Bite',
    SHOT = 'Shot',
}