import { Comment } from '../models/Comment';

export class CommentParser {
    /**
     * Parses commands from the body of a single comment triggered by !!Oregon.
     * If the comment contains only "!!Oregon", it treats it as a "start" command.
     * Additionally, if the command or any argument is a number, it is floored.
     * @param comment A single Comment object to be processed.
     * @returns An object containing the command and its arguments, if any.
     */
    public static parseCommand(comment: Comment): { command: string, args: string[] } {

        if (comment.is_bot) return { command: '', args: [] };
        const commandTrigger: RegExp = /!!Oregon\s*(\w+)?(?:\s+(.*))?/i;
        const match = comment.body.match(commandTrigger);

        if (match) {
            // If the command is exactly "!!Oregon" with no additional command word, set command to "start"
            let command = match[1] ? match[1].toLowerCase() : "start";
            const numCommand = Number(command);
            if (!isNaN(numCommand) && isFinite(numCommand)) {
                command = Math.floor(numCommand).toString();
            }
            const args = match[2] ? match[2].split(' ').map(arg => {
                const numArg = Number(arg);
                return !isNaN(numArg) && isFinite(numArg) ? Math.floor(numArg).toString() : arg;
            }) : [];
            return { command, args };
        }

        // No command found
        return { command: '', args: [] };
    }
}
