import axios, { AxiosInstance, AxiosError, AxiosResponse, AxiosRequestConfig } from 'axios';
import axiosRetry from 'axios-retry';
import Bottleneck from 'bottleneck';
import dotenv from 'dotenv';

// Load environment variables from .env file
dotenv.config();

class SessionManager {
    private static instance: SessionManager;
    public readonly axiosInstance: AxiosInstance;
    private limiter: Bottleneck;

    private constructor() {


        // Initialize the Bottleneck limiter
        this.limiter = new Bottleneck({
            id: "rDramaAPI-Oregon-limiter",
            datastore: "ioredis",
            clearDatastore: true,
            clientOptions: {
                host: process.env.REDIS_HOST!,
                port: Number(process.env.REDIS_PORT),
                password: process.env.REDIS_PASSWORD || undefined, // Use undefined if no password is set
                enableOfflineQueue: true
            },
            // Initial reservoir value set to the maximum daily limit
            reservoir: 1000,

            // Amount to replenish the reservoir every interval
            reservoirIncreaseAmount: 41,

            // Interval for replenishing the reservoir (1 hour in milliseconds)
            reservoirIncreaseInterval: 60 * 60 * 1000, // 3600000 ms

            // Maximum value the reservoir can reach through replenishment
            reservoirIncreaseMaximum: 1000,

            // Safety configurations
            maxConcurrent: 10, // Maximum number of concurrent requests
            minTime: 1000, // Minimum time between dispatches of requests in milliseconds
        });

        this.axiosInstance = axios.create({
            baseURL: 'https://rdrama.net/',
            headers: {
                'Content-Type': 'application/json',
            },
        });

        // Wrap axios requests with the limiter
        this.wrapAxiosInstance(this.axiosInstance);

        axiosRetry(this.axiosInstance, {
            retries: 10,
            retryDelay: this.retryDelayStrategy,
            retryCondition: this.retryCondition,
        });
    }

    /**
     * Retrieves the singleton instance of the SessionManager.
     * @returns {SessionManager} The singleton instance.
     */
    public static getInstance(): SessionManager {
        if (!SessionManager.instance) {
            SessionManager.instance = new SessionManager();
        }
        return SessionManager.instance;
    }

    /**
     * Shuts down the session manager, disconnecting from any external resources.
     * @returns {Promise<void>} A promise that resolves when the shutdown process is complete.
     */
    public async shutdown(): Promise<void> {
        await this.limiter.disconnect();
    }

    private retryDelayStrategy(retryCount: number, error: AxiosError): number {
        const retryAfter = error.response?.headers['retry-after'];
        if (retryAfter) {
            console.log(`429 Retry After: ${retryAfter}`);
            return +retryAfter * 1000;
        }
        return Math.pow(2, retryCount) * 2000;
    }

    private retryCondition(error: AxiosError): boolean {
        // Check for network errors (e.g., socket hang up)
        const isNetworkError = !error.response && Boolean(error.code); // No response and error.code is truthy indicates a network error

        // Check for specific HTTP status codes
        const status = error.response?.status ?? 0;
        const isRateLimitError = status === 429;
        const isServerError = status >= 500;

        // Retry on network errors, rate limit errors, or server errors
        return isNetworkError || isRateLimitError || isServerError;
    }


    private wrapAxiosInstance(instance: AxiosInstance): void {
        // Wrap the get method
        const originalGet = instance.get;
        instance.get = <T = any, R = AxiosResponse<T>>(url: string, config?: AxiosRequestConfig): Promise<R> => {
            return this.limiter.schedule(() => originalGet.apply(instance, [url, config])) as Promise<R>;
        };

        // Wrap the post method
        const originalPost = instance.post;
        instance.post = <T = any, R = AxiosResponse<T>>(url: string, data?: any, config?: AxiosRequestConfig): Promise<R> => {
            return this.limiter.schedule({ priority: 1 }, () => originalPost.apply(instance, [url, data, config])) as Promise<R>;
        };

        // Wrap the put method
        const originalPut = instance.put;
        instance.put = <T = any, R = AxiosResponse<T>>(url: string, data?: any, config?: AxiosRequestConfig): Promise<R> => {
            return this.limiter.schedule({ priority: 1 }, () => originalPut.apply(instance, [url, data, config])) as Promise<R>;
        };

        // Wrap the delete method
        const originalDelete = instance.delete;
        instance.delete = <T = any, R = AxiosResponse<T>>(url: string, config?: AxiosRequestConfig): Promise<R> => {
            return this.limiter.schedule(() => originalDelete.apply(instance, [url, config])) as Promise<R>;
        };
    }

    /**
     * Sets the authorization token for all outgoing requests.
     * @param {string} token - The authorization token to be used.
     */
    public setAuthorizationToken(token: string): void {
        this.axiosInstance.defaults.headers.common['Authorization'] = `${token}`;
    }


}

export default SessionManager;
