import fs from 'fs';
import path from 'path';
import GameState from '../game/gameState';

// Define an enum for message file names
export enum MessageFileName {
    Oregon_WeaponPurchase = 'oregon_WeaponPurchase.txt',
    Oregon_OxenPurchase = 'oregon_OxenPurchase.txt',
    Oregon_FoodPurchase = 'oregon_FoodPurchase.txt',
    Oregon_AmmoPurchase = 'oregon_AmmoPurchase.txt',
    Oregon_ClothingPurchase = 'oregon_ClothingPurchase.txt',
    Oregon_MiscSuppliesPurchase = 'oregon_MiscSuppliesPurchase.txt',
    Oregon_SickInjuredDeath = 'oregon_SickInjuredDeath.txt',
    Oregon_Template = 'oregon_Template.txt',
}

export class MessageService {
    /**
     * Loads messages from a specified text file based on the enum, splitting by a specific delimiter.
     * Each message is separated by '---END---' in the text file.
     * 
     * @param {MessageFileName} fileName - The enum value representing the file containing the messages.
     * @returns {string[] | undefined} An array of messages, or undefined if there was an error loading the messages.
     */
    private static loadMessages(fileName: MessageFileName): string[] | undefined {
        try {
            const messagesPath = path.join(__dirname, '..', 'messages', fileName);
            return fs.readFileSync(messagesPath, 'utf-8').split('---END---').filter(line => line.trim());
        } catch (error) {
            console.error(`Failed to load messages from ${fileName}:`, error);
        }
    }

    /**
     * Selects a random message from the loaded messages and replaces placeholders within it.
     * 
     * @param {MessageFileName} fileName - The enum value representing the file containing the messages to load.
     * @param {Object} placeholders - A mapping of placeholder names to their replacement values.
     * @returns {string | undefined} A formatted message with placeholders replaced, or undefined if messages couldn't be loaded.
     */
    public static getRandomMessage(fileName: MessageFileName, placeholders: { [key: string]: string }): string | undefined {
        const messages = this.loadMessages(fileName);
        if (!messages) return undefined;
        const message = messages[Math.floor(Math.random() * messages.length)];
        return this.replacePlaceholders(message, placeholders);
    }

    /**
     * Replaces placeholders in a message with values from a provided mapping.
     * 
     * @param {string} message - The message containing placeholders.
     * @param {Object} placeholders - A mapping of placeholder names to their replacement values.
     * @returns {string} The message with placeholders replaced by actual values.
     */
    private static replacePlaceholders(message: string, placeholders: { [key: string]: string }): string {
        return Object.keys(placeholders).reduce((acc, key) => {
            const regex = new RegExp(`{${key}}`, 'g');
            return acc.replace(regex, placeholders[key]);
        }, message);
    }

    static formatDate(date: Date): string {
        return date.toLocaleDateString("en-US", {
            weekday: 'long',
            year: 'numeric',
            month: 'long',
            day: 'numeric',
        }).toUpperCase();
    }

    static async statusUpdate(gameState: GameState, message: string): Promise<void> {
        const placeholders: { [key: string]: string } = {
            playerName: gameState.authorName,
            message: message,
            date: gameState.currentDate,
            money: gameState.cashLeftAfterInitialPurchases.toString(),
            totalMileage: gameState.totalMileageWholeTrip.toString(),
            oxen: MessageService.describeOxenQuality(gameState),
            food: gameState.amountSpentOnFood.toString(),
            ammo: gameState.amountSpentOnAmmunition.toString(),
            clothing: gameState.amountSpentOnClothing.toString(),
            supplies: gameState.amountSpentOnMiscellaneousSupplies.toString(),
        };

        console.log(JSON.stringify(placeholders, null, 4))
        //TODO remove Commenting when it is time to go live
        //const parsedMessage = `${MessageService.getRandomMessage(MessageFileName.Oregon_Template, placeholders)}`
        //await CommentPoster.postComment(`c_${this.gameState.comment.id}`, parsedMessage)
    }

    /**
     * Describes the quality of oxen based on the amount spent on animals within the game state.
     * This method assesses the oxen quality by comparing the total amount spent on animals against predefined quality levels.
     *
     * @returns {string} A string describing the quality of the oxen, ranging from 'Basic Quality' to 'Exceptional Quality'. Returns 'Unknown Quality' if the amount spent does not fit the expected range.
     * @example
     * // If the gameState.amountSpentOnAnimals is 250
     * describeOxenQuality(); // returns 'Good Quality'
     */
    static describeOxenQuality(gameState: GameState): string {
        const spent = gameState.amountSpentOnAnimals;
        const qualityLevels = [
            { max: 220, description: 'Basic Quality' },
            { max: 240, description: 'Moderate Quality' },
            { max: 260, description: 'Good Quality' },
            { max: 280, description: 'High Quality' },
            { max: 300, description: 'Exceptional Quality' }
        ];

        for (const level of qualityLevels) {
            if (spent <= level.max) {
                return level.description;
            }
        }

        // Default to the lowest quality if for some reason the amount doesn't fit the expected range
        return 'Unknown Quality';
    }
}


